#==================================================================================================
# package      DE_EPAGES::Paybox::t::unit::PaymentDispatcher
# state        public
#--------------------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::Paybox::API::GatewayInterface::PaymentDispatcher
#==================================================================================================
package DE_EPAGES::Paybox::t::unit::PaymentDispatcher;
use base DE_EPAGES::Core::API::Test;

use strict;

use Test::Exception;
use Test::More;

use DE_EPAGES::Paybox::API::PaymentDispatcher;

use DE_EPAGES::Test::Mock::EpagesObject;

use DE_EPAGES::Paybox::t::unit::MockupObjects qw (
    TEST_ORDER_TOTAL_SENT
    TEST_TRANS_ID
    TEST_TRANS_ID2

    NewMockedShop
);

use DE_EPAGES::Paybox::API::Constants qw (
    STATUS_PAID
    STATUS_FAILED
    OPERATION_ACCEPTED
    OPERATION_REJECTED
    OPERATION_CANCELED
);

use constant TEST_PAYMENT_DATE => '2014-12-31T12:34:56';

#==================================================================================================
# test
#==================================================================================================
sub test {
    my $self = shift;

    $self->test_processPaymentAccepted();
    $self->test_processPaymentCancelled();
    $self->test_processPaymentRejected();

    done_testing();
}

#==================================================================================================
# test_processPaymentAccepted
#
#   Tests the payment process for an ACCEPTED response
#==================================================================================================
sub test_processPaymentAccepted {
    my $self = shift;

    my $SubTestName = 'PaymentDispatcher->test_processPaymentAccepted';

    my $MockedShop = NewMockedShop();
    my $hResult = $self->_test_processPayment($MockedShop, OPERATION_ACCEPTED);

    $self->_checkResult(
        $MockedShop,
        $hResult,
        {
            'SubTestName'           => $SubTestName,
            'ErrorCode'             => undef,
            'Order.PaidOn'          => TEST_PAYMENT_DATE,
            'Order.CancelledOn'     => undef,
            'Payment.TransStatus'   => STATUS_PAID,
            'Payment.TransID2'      => TEST_TRANS_ID2,
            'Payment.TransTimeMod'  => TEST_PAYMENT_DATE,
            'Payment.TransTime'     => TEST_PAYMENT_DATE,
        }
    );

    return;
}

#==================================================================================================
# test_processPaymentCancelled
#
#   Tests the payment process for a CANCEL response
#==================================================================================================
sub test_processPaymentCancelled {
    my $self = shift;

    my $SubTestName = 'PaymentDispatcher->test_processPaymentCancelled';

    my $MockedShop = NewMockedShop();
    my $hResult = $self->_test_processPayment($MockedShop, OPERATION_CANCELED);

    $self->_checkResult(
        $MockedShop,
        $hResult,
        {
            'SubTestName'           => $SubTestName,
            'ErrorCode'             => 'PaymentProcess_Canceled',
            'Order.PaidOn'          => undef,
            'Order.CancelledOn'     => TEST_PAYMENT_DATE,
            'Payment.TransStatus'   => STATUS_FAILED,
            'Payment.TransID2'      => TEST_TRANS_ID2,
            'Payment.TransTimeMod'  => TEST_PAYMENT_DATE,
            'Payment.TransTime'     => TEST_PAYMENT_DATE,
        }
    );

    return;
}

#==================================================================================================
# test_processPaymentRejected
#
#   Tests the payment process for a REJECTED response
#==================================================================================================
sub test_processPaymentRejected {
    my $self = shift;

    my $SubTestName = 'PaymentDispatcher->test_processPaymentRejected';

    my $MockedShop = NewMockedShop();
    my $hResult = $self->_test_processPayment($MockedShop, OPERATION_REJECTED);

    $self->_checkResult(
        $MockedShop,
        $hResult,
        {
            'SubTestName'           => $SubTestName,
            'ErrorCode'             => 'PaymentProcess_Failed',
            'Order.PaidOn'          => undef,
            'Order.CancelledOn'     => TEST_PAYMENT_DATE,
            'Payment.TransStatus'   => STATUS_FAILED,
            'Payment.TransID2'      => TEST_TRANS_ID2,
            'Payment.TransTimeMod'  => TEST_PAYMENT_DATE,
            'Payment.TransTime'     => TEST_PAYMENT_DATE,
        }
    );

    return;
}

#==================================================================================================
# _test_processPayment
#==================================================================================================
sub _test_processPayment {
    my $self = shift;
    my ($MockedShop, $Operation) = @_;

    my $PaymentDispatcher = DE_EPAGES::Paybox::API::PaymentDispatcher->new();
    $PaymentDispatcher = $self->_setMockedMethodsToPaymentDispatcher(
                            $PaymentDispatcher,
                            $MockedShop
                         );

    my $hPaymentContext = {
        'Shop'          => $MockedShop,
        'Operation'     => $Operation,
        'PayboxCode'    => '00000',
        'Amount'        => TEST_ORDER_TOTAL_SENT,
        'TransID'       => TEST_TRANS_ID,
        'TransID2'      => TEST_TRANS_ID2,
        'Authorization' => 'XXXXX'
    };

    return $PaymentDispatcher->processPayment($hPaymentContext);
}

#==================================================================================================
# _setMockedMethodsToPaymentDispatcher
#
#   Wraps in the PaymentDispatcher object the private methods which perform external calls.
#   We replace those methods with a fake version returning a known value.
#==================================================================================================
sub _setMockedMethodsToPaymentDispatcher {
    my $self = shift;
    my ($PaymentDispatcher, $MockedShop) = @_;

    my $MockedOrder = $MockedShop->get('Orders')->[0];
    my $MockedLinteItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');

    my $MockPaymentDispatcher = Test::MockObject::Extends->new($PaymentDispatcher);
    $MockPaymentDispatcher->mock(
        '_ExistsObjectByGUID',
        sub { return 1; }
    );
    $MockPaymentDispatcher->mock(
        '_LoadObjectByGUID',
        sub { return $MockedLinteItemPayment; }
    );
    $MockPaymentDispatcher->mock(
        '_GetCurrentDateTime',
        sub { return TEST_PAYMENT_DATE; }
    );
    $MockPaymentDispatcher->mock(
        '_Basket2Order',
        sub {
            my $self = shift;
            my ($BasketOrder) = @_;
            return $BasketOrder;
        }
    );
    $MockPaymentDispatcher->mock(
        '_LogPayment',
        sub { return ; }
    );

    return $MockPaymentDispatcher;
}

#==================================================================================================
# _checkResult
#
#   Checks all the attributes in the Order or LineItemPayment that could be modified in a payment
#   process. The attributes are tested against expected values
#==================================================================================================
sub _checkResult {
    my $self = shift;
    my ($MockedShop, $hResult, $hExpected) = @_;

    my $SubTestName = $hExpected->{'SubTestName'};

    my $MockedOrder = $MockedShop->get('Orders')->[0];
    my $MockedLinteItemPayment = $MockedOrder->get('LineItemContainer')->get('Payment');

    my $ErrorCode = $hExpected->{'ErrorCode'};
    is( $hResult->{'ErrorCode'},
        $ErrorCode,
        "$SubTestName - ErrorCode is '$ErrorCode'. OK"
    );

    my $OrderPaid = $hExpected->{'Order.PaidOn'};
    is( $MockedOrder->get('PaidOn'),
        $OrderPaid,
        "$SubTestName - Order->PaidOn was assigned OK"
    );

    my $OrderCancelled = $hExpected->{'Order.CancelledOn'};
    is( $MockedOrder->get('CancelledOn'),
        $OrderCancelled,
        "$SubTestName - Order->CancelledOn was assigned OK"
    );

    my $PaymentTrasnStatus = $hExpected->{'Payment.TransStatus'};
    is( $MockedLinteItemPayment->get('TransStatus'),
        $PaymentTrasnStatus,
        "$SubTestName - Payment->TransStatus was assigned OK"
    );

    my $PaymentTransID2 = $hExpected->{'Payment.TransID2'};
    is( $MockedLinteItemPayment->get('TransID2'),
        $PaymentTransID2,
        "$SubTestName - Payment->TransID2 was assigned OK"
    );

    my $PaymentTransTimeMod = $hExpected->{'Payment.TransTimeMod'};
    is( $MockedLinteItemPayment->get('TransTimeMod'),
        $PaymentTransTimeMod,
        "$SubTestName - Payment->TransTimeMod was assigned OK"
    );

    my $PaymentTransTime = $hExpected->{'Payment.TransTime'};
    is( $MockedLinteItemPayment->get('TransTime'),
        $PaymentTransTime,
        "$SubTestName - Payment->TransTime was assigned OK"
    );

    return;
}

#==================================================================================================
# main
#==================================================================================================
__PACKAGE__->runScript();

